/******************************************************************************
Utility to convert a LTP flow to a TCP flow for LTP analysis via tcptrace.
LTP Header information

Copyright (C) 2013  Samuel Jero <sj323707@ohio.edu>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

Author: Samuel Jero <sj323707@ohio.edu>
Date: 06/2013

Notes:
	1)Only handles one LTP "connection". There isn't a good way to separate
		different LTP "connections" from new sessions of the same "connection".
		Use Tcpdump filters to separate connections. Libpcap filtering could also
		be added in ltp2tcp.
	2)Uses some special types from Linux (u_char, u_int32_t)
******************************************************************************/

#ifndef LTP_H_
#define LTP_H_
#include <stdlib.h>

/* structure to hold all the data from an LTP header.
 *  Note that this isn't the RAW header like in IP,TCP;
 *  it is a structure to hold all the information. The
 *  RAW header is mostly SDNV values
 */
struct ltp_hdr_d{
	u_char 	vers_ctl;		/* Version and Control Field*/
	int 	sender_id;		/* Session sender id*/
	int 	session_num;	/* session number */
	u_char 	extensions;		/* number of extensions field (split header/trailer)*/
	int 	cls_id;			/* Data segment Client Service ID */
	int 	offset;			/* Data segment data offset*/
	int 	length;			/* Data segment data length*/
	int 	checkpoint;		/* Checkpoint number */
	int 	report;			/* Report Number */
	int 	ubound;			/* Report Upper Bound*/
	int 	lbound;			/* Report Lower Bound*/
	int 	rclaims;		/* Number of Reception Claims for Report */
	int 	reason;			/* Cancellation Reason code */
};

/* SDNV manipulation functions*/
int evaluate_sdnv(const u_char* loc, int *bytecount);
long long evaluate_sdnv_64(const u_char* loc, int *bytecount);

#endif
